from sweetbean.stimulus.Stimulus import _KeyboardResponseStimulus
from sweetbean.variable import DataVariable, FunctionVariable


class HtmlKeyboardResponse(_KeyboardResponseStimulus):
    """
    A base class for html stimuli
    """

    type = "jsPsychHtmlKeyboardResponse"

    def __init__(
        self,
        duration=None,
        stimulus="",
        choices=None,
        correct_key="",
        side_effects=None,
    ):
        if choices is None:
            choices = []
        super().__init__(locals(), side_effects)

    def _add_special_param(self):
        pass

    def _set_before(self):
        pass


class Text(HtmlKeyboardResponse):
    """
    Show colored text
    """

    l_template: str = (
        "You see "
        '{% if text %}"{{text}}" in "{{color}}"'
        "{% else %}a blank screen{% endif %}"
        "{% if duration %} for {{duration}}ms{% endif %}."
    )

    def __init__(
        self,
        duration=None,
        text="",
        color="white",
        choices=None,
        correct_key="",
        side_effects=None,
    ):
        """
        Arguments:
            duration: time in ms the stimulus is presented
            text: the text should be presented
            color: the color of the text
            choices: the keys that will be recorded if pressed
            correct_key: the correct key to press
            side_effects: a dictionary of side effects
        """
        if choices is None:
            choices = []

        def stim(cl, txt):
            return f"<div style='color:{cl}'>{txt}</div>"

        stimulus_ = FunctionVariable("stimulus", stim, [color, text])
        super().__init__(duration, stimulus_, choices, correct_key, side_effects)

        self.arg.update({"text": text, "color": color})


Stroop = Text


class Blank(HtmlKeyboardResponse):
    """
    shows a blank screen
    """

    def __init__(
        self,
        duration=None,
        choices=None,
        correct_key="",
        side_effects=None,
    ):
        """
        Arguments:
            duration: time in ms the stimulus is presented
            choices: the keys that will be recorded if pressed
            correct_key: the correct key to press
            side_effects: a dictionary of side effects
        """
        super().__init__(
            duration=duration,
            stimulus="",
            choices=choices,
            correct_key=correct_key,
            side_effects=side_effects,
        )


class Fixation(Text):
    """
    show a white cross in the middle of the screen
    """

    def __init__(self, duration=None, side_effects=None):
        """
        Arguments:
            duration: time in ms the stimulus is presented
            side_effects: a dictionary of side effects
        """
        super().__init__(
            duration=duration,
            text="+",
            color="white",
            choices=[],
            correct_key="",
            side_effects=side_effects,
        )


class Feedback(Text):
    """
    show the word correct or incorrect dependent on a correct response to a stimulus before
    """

    def __init__(
        self,
        duration=None,
        correct_message="Correct!",
        false_message="False!",
        correct_color="green",
        false_color="red",
        window=1,
        side_effects=None,
    ):
        """
        Arguments:
            duration: time in ms the stimulus is presented
            correct_message: the message to show if the response was correct
            false_message: the message to show if the response was false
            correct_color: the color of the message if the response was correct
            false_color: the color of the message if the response was false
            window: how far back is the stimulus to check
                    (that stimulus needs to have a choice and a correct_key parameter)
            side_effects: a dictionary of side effects
        """
        correct = DataVariable("correct", window)

        feedback_txt = FunctionVariable(
            "feedback_txt",
            lambda c_msg, f_msg, cor: c_msg if cor else f_msg,
            [correct_message, false_message, correct],
        )

        feedback_color = FunctionVariable(
            "feedback_color",
            lambda c_color, f_color, cor: c_color if cor else f_color,
            [correct_color, false_color, correct],
        )

        super().__init__(
            duration,
            feedback_txt,
            feedback_color,
            choices=[],
            correct_key="",
            side_effects=side_effects,
        )


class Flanker(Text):
    """
    show a flanker stimulus (<< < <<; << > <<, >> < >>, ...)
    """

    def __init__(
        self,
        duration=None,
        direction="left",
        distractor="left",
        choices=None,
        correct_key="",
        color="white",
        n_flankers=2,
        side_effects=None,
    ):
        """
        Arguments:
            duration: time in ms the stimulus is presented
            direction: the direction of the target (allowed: left, right, l, r, L, R)
            distractor: the direction of the distractor (allowed: left, right, l, r, L, R)
            choices: the keys that will be recorded if pressed
            correct_key: the correct key to press
            color: the color of the text
            n_flankers: the number of distractors
            side_effects: a dictionary of side effects
        """

        def _txt(dr, dst, n):
            normalized_dir = dr.lower()
            normalized_dist = dst.lower()
            t = ""
            d = ""
            if normalized_dir == "right" or normalized_dir == "r":
                t = ">"
            if normalized_dir == "left" or normalized_dir == "l":
                t = "<"
            if normalized_dist == "right" or normalized_dist == "r":
                d = ">"
            if normalized_dist == "left" or normalized_dist == "l":
                d = "<"
            d = d * n
            return f"{d}{t}{d}"

        txt = FunctionVariable("target", _txt, [direction, distractor, n_flankers])

        super().__init__(
            duration=duration,
            text=txt,
            color=color,
            choices=choices,
            correct_key=correct_key,
            side_effects=side_effects,
        )


class Symbol(HtmlKeyboardResponse):
    """
    show a symbol
    """

    def __init__(
        self,
        duration=None,
        symbol="",
        color="white",
        choices=None,
        correct_key="",
        side_effects=None,
    ):
        """
        Arguments:
            duration: time in ms the stimulus is presented
            symbol: the symbol to show (allowed: square, triangle, circle)
            color: the color of the symbol
            choices: the keys that will be recorded if pressed
            correct_key: the correct key to press
            side_effects: a dictionary of side effects
        """

        def stim(symbl, clr):
            return (
                f"<div class='sweetbean-{symbl}' style='background-color:{clr}'></div>"
            )

        stimulus_ = FunctionVariable("stimulus", stim, [symbol, color])
        super().__init__(duration, stimulus_, choices, correct_key, side_effects)



"""
After the participant is greeted with 'Welcome! We show a survey. Press SPACE to continue' until
they press the spacebar, they are asked two multiple-choice questions. One is about how they are,
which they can rate as bad, good, or fine. The other is about their handedness.
"""

from sweetbean import Block, Experiment
from sweetbean.stimulus import MultiChoiceSurvey, Text

# EVENT SEQUENCE

stim_1 = Text(
    text="Welcome! We show a survey. Press SPACE to continue",
    choices=[" "],
)

stim_2 = MultiChoiceSurvey(
    questions=[
        {"prompt": "How are you?", "options": ["bad", "good", "fine"]},
        {
            "prompt": "What is your handedness?",
            "options": ["left", "right", "other", "prefer not to say"],
        },
    ]
)

event_sequence = [stim_1, stim_2]

# BLOCK DESIGN

block = Block(event_sequence)
experiment = Experiment([block])
experiment.to_html("multi_choice.html")



# -*- coding: utf-8 -*-
""" Weber Fechner

An example for Weber Fechner.  Here, we have a simple stimulus timeline:
[fixation, blank, rdp, blank, feedback]

"""

from sweetbean import Block, Experiment
from sweetbean.stimulus import Blank, Feedback, Fixation, RandomDotPatterns
from sweetbean.variable import TimelineVariable

timeline = [
    {"S1": 40, "S2": 70, "correct_key": "y"},
    {"S1": 70, "S2": 70, "correct_key": "n"},
    {"S1": 70, "S2": 40, "correct_key": "y"},
    {"S1": 70, "S2": 70, "correct_key": "n"},
    {"S1": 40, "S2": 70, "correct_key": "y"},
    {"S1": 70, "S2": 40, "correct_key": "y"},
    {"S1": 40, "S2": 40, "correct_key": "n"},
    {"S1": 40, "S2": 40, "correct_key": "n"},
    {"S1": 40, "S2": 40, "correct_key": "n"},
    {"S1": 70, "S2": 40, "correct_key": "y"},
]

fixation = Fixation(800)

blank_1 = Blank(400)
blank_2 = Blank(1000)

s1 = TimelineVariable("S1")
s2 = TimelineVariable("S2")
correct_key = TimelineVariable("correct_key")

# We can use these variables in the stimuli declaration:

rdp = RandomDotPatterns(
    duration=1000,
    number_of_oobs=[s1, s2],
    number_of_apertures=2,
    choices=["y", "n"],
    correct_key=correct_key,
)


feedback = Feedback(500, window=2)

event_sequence = [fixation, blank_1, rdp, blank_2, feedback]

block = Block(event_sequence, timeline)

# Create an image of the stimuli sequence of the block
block.to_image(
    "weber_fechner.png",
    data=[None, None, {"correct": True}, None, None],
    zoom_factor=[3, 3, 1, 3, 3],
    sequence=True,
)

# Create an HTML file of the experiment
experiment = Experiment([block])
experiment.to_html("weber_fechner.html")



from sweetbean import Block, Experiment
from sweetbean.stimulus import Image

stim_1 = Image(
    stimulus="https://media.istockphoto.com/id/120492078/photo/"
    "banana.jpg?s=1024x1024&w=is&k=20&c=M9KLVNgqLft_btWgSu0iZAmdv2asI11Qel-6fsQK140=",
    choices=[" "],
)

block = Block([stim_1])

# Create an image of the stimulus
block.to_image("./", data=None, sequence=False, timeline_idx="random", zoom_factor=1)

# Create an HTML file of the experiment
experiment = Experiment([block])
experiment.to_html("image.html")



from jinja2 import Template


def demographic(age, gender):
    """
    Creates a personalized message using age and gender.

    Parameters:
        age (int): The age of the individual.
        gender (str): The gender of the individual (e.g., 'man', 'woman').

    Returns:
        str: A formatted message based on the given demographics.

    Example:
        >>> demographic(52, 'woman')
        'You are a 52-year-old woman.'

    """

    # Template for generating the message
    template = "You are a {{ age }}-year-old {{ gender }}."

    # Render the template with local variables
    return Template(template).render(age=age, gender=gender)



"""
This script demonstrates how to use the `run_on_language` method to run an experiment
with image stimuli.
"""

from sweetbean import Block, Experiment
from sweetbean.stimulus import Image

stim_1 = Image(
    stimulus="https://media.istockphoto.com/id/120492078/photo/"
    "banana.jpg?s=1024x1024&w=is&k=20&c=M9KLVNgqLft_btWgSu0iZAmdv2asI11Qel-6fsQK140=",
    choices=[" "],
)

block = Block([stim_1])
experiment = Experiment([block])

# To make this work, there is a file called image_prompts.json in the same directory as this
# script with the following content:
# {
# "https://media.istockphoto.com/id/120492078/photo/banana.jpg?s=1024x1024&w=is&k=20&c=M9KLVNgqLft_btWgSu0iZAmdv2asI11Qel-6fsQK140=": "You see a picture of a banana."   # noqa: E501
# }

experiment.run_on_language()



"""
After the participant is greeted with 'Welcome! We show a survey. Press SPACE to continue' until
they press the spacebar, they are asked free-from questions about how old they are, their gender,
and their handedness.
"""

from sweetbean import Block, Experiment
from sweetbean.stimulus import Text, TextSurvey

# EVENT SEQUENCE

stim_1 = Text(
    text="Welcome! We show a survey. Press SPACE to continue",
    choices=[" "],
)

stim_2 = TextSurvey(
    questions=["How old are you?", "What is your gender?", "What is your handedness?"]
)

event_sequence = [stim_1, stim_2]

# BLOCK DESIGN

block = Block(event_sequence)
experiment = Experiment([block])
experiment.to_html("text_survey.html")



from sweetbean.stimulus.Stimulus import _BaseStimulus


class _Template_(_BaseStimulus):
    """
    A template for creating a new stimulus.
    The following methods are probably sufficient to implement.
    """

    def __init__(self, args, side_effects=None):
        """
        Change this to match the arguments of the class
        """
        super().__init__(args, side_effects)

    def process_l(self, prompts, get_input, multi_turn, datum):
        """
        This is used to process the arguments, generate a prompt and
        get a response from language input
        """
        data = self.l_args.copy()
        # Add your code for prompt generation here:

        # Add your code for processing the response and adding to data here:
        return data, prompts

    def _add_special_param(self):
        """
        This can be used to add special parameters to the js object that are
        not input arguments of the class
        """
        pass

    def _process_response(self):
        """
        This can be used to process the responses given by the participant and
        adding data to the js object
        """
        pass

    def _set_before(self):
        """
        This can be used to set global parameters before the stimulus is presented
         (for example css variables)
        """
        pass



import math

from sweetbean.stimulus.Stimulus import _BaseStimulus
from sweetbean.util.parse import to_js
from sweetbean.variable import FunctionVariable


class HtmlChoice(_BaseStimulus):
    """
    html elements that can be used to present a choice with a mouse press
    """

    type = "jsPsychHtmlChoice"

    def __init__(
        self,
        duration=None,
        html_array=None,
        values=None,
        time_after_response=3000,
        side_effects=None,
    ):
        """
        Arguments:
            duration (int): The duration of the stimulus
            html_array (list): An array of html elements that can be clicked
            values (list): An array of values corresponding to the html elements
            time_after_response (int): The time after a response is made
                (for example, for animations)
            side_effects (dict): A dictionary of side effects
        """
        if values is None:
            values = []
        if html_array is None:
            html_array = []
        super().__init__(locals(), side_effects)

    def _add_special_param(self):
        pass

    def _process_response(self):
        self.js_data += 'data["bean_value"] = data["value"];'
        self.js_data += 'data["bean_response"] = data["choice"];'

    def _set_before(self):
        pass

    def process_l(self, prompts, get_input, multi_turn):
        raise NotImplementedError


class Bandit(HtmlChoice):
    """
    A bandit task.
    """

    type = "jsPsychHtmlChoice"

    def __init__(
        self,
        duration=None,
        bandits=None,
        time_after_response=2000,
        side_effects=None,
    ):
        """
        Arguments:
            duration (int): The duration of the stimulus
            bandits (list): A list of bandits
                (in the form of dictionaries with entries for color and value)
            time_after_response (int): The time after a response is made
                (for example, for animations)
            side_effects (dict): A dictionary of side effects
        """
        if bandits is None:
            bandits = []

        def bandits_to_html(bdts):
            n = len(bdts)
            cols = math.ceil(math.sqrt(n))
            rows = math.ceil(n / cols)

            # Calculate square size
            square_width = (100 - (cols + 1) * 10) / cols
            square_height = (100 - (rows + 1) * 10) / rows
            square_size = min(square_width, square_height)

            # Generate square positions
            positions = []
            for r in range(rows):
                # Calculate the number of squares in this row
                if r == rows - 1 and n % cols != 0:  # Last row and it's not full
                    row_squares = n % cols
                else:
                    row_squares = cols

                # Calculate the starting x position to center the row
                total_row_width = row_squares * square_size + (row_squares - 1) * 10
                start_x = (100 - total_row_width) / 2

                for c in range(row_squares):
                    x = start_x + c * (square_size + 10)
                    y = 10 + r * (square_size + 10)
                    positions.append((x, y, square_size))
            html = []
            for bdt, p in zip(bdts, positions):
                html.append(
                    f'<div class="slotmachine" '
                    f'style="position: absolute; top:{p[1]}vh; left:{p[0]}vw; '
                    f'width: {p[2]}vw; height: {p[2]}vh; border-color: {bdt["color"]}">'
                    f"</div>"
                )
            return html

        def vals(bdts):
            return [b["value"] for b in bdts]

        html_array = FunctionVariable("html_array", bandits_to_html, [bandits])
        values = FunctionVariable("values", vals, [bandits])

        super().__init__(
            duration=duration,
            html_array=html_array,
            values=values,
            time_after_response=time_after_response,
            side_effects=side_effects,
        )
        self.arg.update({"bandits": bandits})

    def _set_before(self):
        res = "const root=document.documentElement;"
        res += (
            f'root.style.setProperty("--slotmachine-time",'
            f'`${{({to_js(self.arg_js["time_after_response"])})/2}}ms`);'
        )
        res += (
            f'root.style.setProperty("--slotmachine-time-after",'
            f'`${{({to_js(self.arg_js["time_after_response"])})/2}}ms`);'
        )
        self.js_before = f"on_load:()=>{{{res}}},"

    def process_l(self, prompts, get_input, multi_turn, datum=None):
        current_prompt = f' You see {len(self.l_args["bandits"])} bandits.'
        for idx, bandit in enumerate(self.l_args["bandits"]):
            current_prompt += f' Bandit {idx + 1} is {bandit["color"]}.'
        current_prompt += (
            " Choose a bandit by naming the number of the bandit. You name "
        )
        if not multi_turn:
            in_prompt = " ".join([p for p in prompts]) + current_prompt + "<<"
        else:
            in_prompt = current_prompt + "<<"
        rest_data = None
        if not datum:
            _r = get_input(in_prompt)
            if isinstance(_r, str):
                response = _r
            elif isinstance(_r, dict):
                if "response" not in _r:
                    raise Exception(f"{_r} has an invalid response format")
                response = _r["response"]
                _r.pop("response")
                rest_data = _r

            else:
                raise Exception(f"{_r} has an invalid response format")
            # response = get_input(in_prompt)
        else:
            response = datum["response"] + 1
        if int(response) < 1 or int(response) > len(self.l_args["bandits"]):
            prompts.append(
                current_prompt + f"<<{response}>>. " f"The response was invalid."
            )
            value = 0
        else:
            prompts.append(
                current_prompt + f"<<{response}>>. "
                f"The value of the chosen bandit was {self.l_args['values'][int(response) - 1]}."
            )
            value = self.l_args["values"][int(response) - 1]
        data = self.l_args.copy()
        data.update(
            {
                "response": int(response) - 1,
                "value": value,
            }
        )
        if rest_data:
            data.update(rest_data)
        return data, prompts



"""
(only in pr-release)
An instructions screen welcoming the participant to the experiment,
followed by a second page of instructions, and a final page of instructions.
"""

from sweetbean import Block, Experiment
from sweetbean.stimulus import Generic

instructions = Generic(
    type="jsPsychInstructions",
    pages=[
        "Welcome to the experiment",
        "This is the second page of instructions",
        "This is the final page",
    ],
    show_clickable_nav=True,
)

block = Block([instructions])

experiment = Experiment([block])
experiment.to_html("instructions.html")



import warnings

from sweetbean.stimulus.Stimulus import _BaseStimulus


class Generic(_BaseStimulus):
    """
    A generic stimulus.

    Attention: This stimulus implements jsPsych stimuli but can not be
        used to run experiments on language
    """

    _not_tested: bool = False
    _js_plugins: list = []

    def __init__(
        self,
        side_effects=None,
        **kwargs,
    ):
        if "type" not in kwargs:
            raise ValueError("`type` argument must be defined in Generic stimulus.")
        if not Generic._not_tested:
            warnings.warn(
                "You are using a Generic Stimulus. This feature has not been fully tested.\n"
                "It might work for creating html files or javascript functions but probably\n"
                "will not work with running experiments on language."
            )
            Generic._not_tested = True
        if not kwargs["type"] in Generic._js_plugins:
            warnings.warn(
                f"You are using a jsPsych plugin {kwargs['type']} with a generic class.\n"
                f"Make sure to add the script plugin to the generated HTML file\n"
                f"or javascript function. You can find a list here:\n"
                f"https://www.jspsych.org/v7/plugins/list-of-plugins/\n"
            )
            Generic._js_plugins.append(kwargs["type"])

        self.side_effects = side_effects
        if "self" in kwargs:
            del kwargs["self"]
        if "__class__" in kwargs:
            del kwargs["__class__"]
        if "side_effects" in kwargs:
            del kwargs["side_effects"]

        self.arg = kwargs
        self.arg_js = {}
        if "duration" in self.arg:
            self.arg_js["trial_duration"] = self.arg["duration"]
        for key in self.arg:
            self.arg_js[key] = kwargs[key]

    def _add_special_param(self):
        pass

    def _process_response(self):
        pass

    def _set_before(self):
        pass



from sweetbean.stimulus.Stimulus import _BaseStimulus
from sweetbean.variable import FunctionVariable


class _Survey(_BaseStimulus):
    """
    A base class for surveys
    """

    def __init__(self, questions=None, side_effects=None):
        super().__init__(locals(), side_effects=side_effects)

    def _add_special_param(self):
        pass

    def _process_response(self):
        self.js_data += 'data["bean_response"]=data["response"];'

    def _set_before(self):
        pass


class TextSurvey(_Survey):
    """
    A survey that asks for text input
    """

    type = "jsPsychSurveyText"

    def __init__(self, questions=None, side_effects=None):
        """
        Arguments:
            questions: a list of strings representing the questions
            side_effects: a dictionary of side effects
        """
        if not questions:
            questions = []

        def get_prompts(_prompts):
            prompts_ = []
            for p in _prompts:
                prompts_.append({"prompt": p})
            return prompts_

        questions_ = FunctionVariable("questions", get_prompts, [questions])
        super().__init__(questions_, side_effects=side_effects)

    def process_l(self, prompts, get_input, multi_turn, datum=None):
        current_prompt = []
        responses = {}
        data = self.l_args.copy()
        for idx, question in enumerate(self.l_args["questions"]):
            current_prompt.append(question["prompt"])
            if not multi_turn:
                _in_prompt = (
                    " ".join([p for p in prompts])
                    + " ".join([c for c in current_prompt])
                    + "<<"
                )
            else:
                _in_prompt = current_prompt[-1] + "<<"
            if not datum:
                response = get_input(_in_prompt)
            else:
                response = datum["response"][f"Q{str(idx)}"]
            current_prompt[-1] += f"<<{response}>>"
            responses[f"Q{str(idx)}"] = response
        data.update({"response": responses})
        prompts += current_prompt
        return data, prompts


class MultiChoiceSurvey(_Survey):
    """
    A survey that asks for multiple choice input
    """

    type = "jsPsychSurveyMultiChoice"

    def __init__(self, questions=None, side_effects=None):
        """
        Arguments:
            questions: a list of dictionaries with the keys "prompt" and "options"
            side_effects: a dictionary of side effects
        """
        if not questions:
            questions = []

        def get_prompts(_prompts):
            prompts_ = []
            for p in _prompts:
                prompts_.append({"prompt": p["prompt"], "options": p["options"]})
            return prompts_

        questions_ = FunctionVariable("questions", get_prompts, [questions])
        super().__init__(questions_, side_effects=side_effects)

    def process_l(self, prompts, get_input, multi_turn, datum=None):
        current_prompt = []
        responses = {}
        data = self.l_args.copy()
        for idx, question in enumerate(self.l_args["questions"]):
            current_prompt.append(question["prompt"])
            current_prompt[-1] += " Your options are: " + ", ".join(
                [str(i) for i in question["options"]]
            )
            if not multi_turn:
                _in_prompt = (
                    " ".join([p for p in prompts])
                    + " ".join([c for c in current_prompt])
                    + "<<"
                )
            else:
                _in_prompt = current_prompt[-1] + "<<"
            if not datum:
                response = get_input(_in_prompt)
            else:
                response = datum["response"][f"Q{str(idx)}"]
            current_prompt[-1] += f"<<{response}>>"
            responses[f"Q{str(idx)}"] = response
        data.update({"response": responses})
        prompts += current_prompt
        return data, prompts


#
#
class LikertSurvey(_Survey):
    """
    A survey that asks for Likert scale input
    """

    type = "jsPsychSurveyLikert"

    def __init__(self, questions=None, side_effects=None):
        """
        Arguments:
            questions: a list of dictionaries with the keys "prompt" and "labels"
            side_effects: a dictionary of side effects
        """
        if not questions:
            questions = []

        def get_prompts(_prompts):
            prompts_ = []
            for p in _prompts:
                prompts_.append({"prompt": p["prompt"], "labels": p["labels"]})
            return prompts_

        questions_ = FunctionVariable("questions", get_prompts, [questions])
        super().__init__(questions_, side_effects=side_effects)

    #
    @classmethod
    def from_scale(cls, prompts=None, scale=None, side_effects=None):
        """
        Create a LikertSurvey from a scale

        Arguments:
            prompts: a list of strings representing the prompts
            scale: a list of strings representing the scale
            side_effects: a dictionary of side effects
        """
        if not prompts:
            prompts = []
        if not scale:
            scale = []
        prompts_ = []
        for p in prompts:
            prompts_.append({"prompt": p, "labels": scale})
        return cls(prompts_, side_effects=side_effects)

    def process_l(self, prompts, get_input, multi_turn, datum=None):
        current_prompt = []
        responses = {}
        data = self.l_args.copy()
        for idx, question in enumerate(self.l_args["questions"]):
            current_prompt.append(question["prompt"])
            current_prompt[-1] += " Your options are: " + ", ".join(
                [str(i) for i in question["labels"]]
            )
            if not multi_turn:
                _in_prompt = (
                    " ".join([p for p in prompts])
                    + " ".join([c for c in current_prompt])
                    + "<<"
                )
            else:
                _in_prompt = current_prompt[-1] + "<<"
            if not datum:
                response = get_input(_in_prompt)
            else:
                response = datum["response"][f"Q{str(idx)}"]
            current_prompt[-1] += f"<<{response}>>"
            responses[f"Q{str(idx)}"] = response
        data.update({"response": responses})
        prompts += current_prompt
        return data, prompts



def process_js(js_data):
    """
    Process the data from a SweetBean generated jsPsych experiment.
    """
    res = []
    for d in js_data:
        res_dict = {}
        for key, value in d.items():
            if key in ["rt", "stimulus", "type", "response"]:
                res_dict[key] = value
            if key.startswith("bean_"):
                res_dict[key[5:]] = value
        res.append(res_dict)
    return res


def get_n_responses(data):
    """
    Get the number of responses in the data.
    """
    n = 0
    for d in data:
        if "response" in d and d["response"] is not None:
            n += 1
    return n


def until_response(data, n):
    """
    Get the data until the nth response.
    (This is helpful, for example, to get the data up a certain point
    and then run the rest of the experiment with `run_on_language`)
    """
    i = 0
    for idx, d in enumerate(data):
        if "response" in d and d["response"] is not None:
            i += 1
        if i > n:
            return data[:idx]



from sweetbean import Block, Experiment
from sweetbean.stimulus import Bandit, Text
from sweetbean.variable import (
    DataVariable,
    FunctionVariable,
    SharedVariable,
    SideEffect,
    TimelineVariable,
)

timeline = [
    {
        "bandit_1": {"color": "orange", "value": 10},
        "bandit_2": {"color": "blue", "value": 0},
        "bandit_3": {"color": "red", "value": 5},
    },
    {
        "bandit_1": {"color": "orange", "value": 9},
        "bandit_2": {"color": "blue", "value": 1},
        "bandit_3": {"color": "red", "value": 5},
    },
    {
        "bandit_1": {"color": "orange", "value": 8},
        "bandit_2": {"color": "blue", "value": 2},
        "bandit_3": {"color": "red", "value": 5},
    },
    {
        "bandit_1": {"color": "orange", "value": 7},
        "bandit_2": {"color": "blue", "value": 3},
        "bandit_3": {"color": "red", "value": 5},
    },
    {
        "bandit_1": {"color": "orange", "value": 6},
        "bandit_2": {"color": "blue", "value": 4},
        "bandit_3": {"color": "red", "value": 5},
    },
]

bandit_1 = TimelineVariable("bandit_1")
bandit_2 = TimelineVariable("bandit_2")
bandit_3 = TimelineVariable("bandit_3")

score = SharedVariable("score", 0)
value = DataVariable("value", 0)

update_score = FunctionVariable(
    "update_score", lambda sc, val: sc + val, [score, value]
)

update_score_side_effect = SideEffect(score, update_score)

bandit_task = Bandit(
    bandits=[bandit_1, bandit_2, bandit_3],
    side_effects=[update_score_side_effect],
)
show_score = Text(duration=1000, text=score)

block = Block([bandit_task, show_score], timeline=timeline)

# Create an image of the stimuli sequence of the block
block.to_image("bandit.png", [{"value": 5}, None], zoom_factor=[1, 3])

# Create HTML file of the experiment
experiment = Experiment([block])
experiment.to_html("bandit.html")
